
run('EB_2d_modellv2.m');

%%%%%% simulation time %%%%%%%%
sim_time = 300;
dt = 0.1;
t = 0:dt:sim_time;

n_meas = 6;
vnoise = randn(n_meas,length(t));
n_meas2 = n_meas - 2;
vnoise2 = randn(n_meas2+1,length(t));
%% CALCULATE WIND SPEED REALIZATION
U_10min = 9; %avg 10 min windspeed at hub 
f_freq = (0:0.001:1)';
t_int = 0.01; % turbulence intensity
[S_kaim, S_white] = kaim_wnoisespectra(U_10min,f_freq,t_int);
zref = 100;
uref = U_10min; %wind speed at reference height zref
[uw,turb] = kaimal_realizationV2(t,uref,zref,S_kaim,f_freq);

D_bl = 89*2; %diameter of blades
A_blades = (pi*D_bl^2)/4; % Swept blade area
rho_a = 1.205; %density air
C_T = 0.45;

%%  CALCULATE 3P FORCES

omega_r = 9.6;
%tau_3PV1 = loading3P(U_10min,w_radius.dy,N_tower,lel_tower,t,omega_r);
tau_3P= loading3PV2(U_10min,w_radius.dy,N_tower,lel_tower,t,omega_r,uw);

tau_3P(end-1,:) = [];
tau_3P(end,:) = [];
tau_3P(1:3:end,:) = [];
tau_3P(2:2:end,:) = 0;


b2 = zeros(size(M,1),size(tau_3P,1));
bstart =2*( N_mono3 + N_mono2 + N_mono1 + N_found);
for i = 0:size(tau_3P,1)-1
b2(bstart + i,i+1) = 1;
end

%% CALCULATE WAVE FORCES
%Give enviromental data to calculate jonswap spectrum
Hm0=1;Tp=5; gam=0; sa=0.07; sb=0.09; A=-1;% 
envdata=[Hm0 Tp gam sa sb A];
 Diameter = w_radius.dy(1); % diameter of structure
 
tau_waves = waveloadsschultzV3(envdata,t,N_found,N_mono1);




tau_waves2 = tau_waves;

%%% input matrix for wave forces %%%%
b1 = zeros(size(M,1),size(tau_waves,1));
bstart = dof*(N_mono3+N_mono2);
b1(bstart:bstart + size(tau_waves,1)-1,:) = eye(size(tau_waves,1)); 
clear bstart


%% CALCULATE CURRENT FORCES
Vc = 1; %current velocity
dof = 3; % only assuming the transversal and rotational nodes

tau_curr = current_force(Diameter,t,dof,N_mono1,N_found,Vc,lel_found,lel_mono1);
tau_curr(1:3:end,:) = [];
tau_curr(4:2:end-2,:) = 0;
%%
no_input = 1;
b=zeros(size(M,1),no_input); % input for input_state estimator
b(end-1,1) = 1;
% b(2*41,2) = 1;
%%% OUTPUT VECTOR %%%
N_tot = N_mono3 + N_mono2 + N_mono1 + N_found + N_tower;
%  C = eye(size(M));
C = zeros(n_meas2,size(M,1));
C(1,end-1) = 1;
C(2,2*65+1) = 1;
C(3,2*45+1) = 1;
C(4,2*41+1) = 1;



%% INPUT STATE ESTIMATOR NO.1 

no_modes =10;

c2 = zeros(n_meas-n_meas2,size(M,1));
c2(1,2*41+1) = 1;
c2(2,end-1) = 1;


[A_est1,eigVest,G_est,J_est,T_est1,eigval_est1] = estimator_discrp(M,K,D,C,b,no_modes,c2);
phiest1 = expm(A_est1*dt);
B_est = [zeros(size(phiest1,2)/2,no_input);eigVest'*b];
gamma_est = A_est1\(phiest1 - eye(size(A_est1)))*B_est;

Eb1 = eye(size(phiest1));

Ebd1 = A_est1\(phiest1 -eye(size(phiest1)))*Eb1;


%% TRUE MODEL %%
[eigV eigval] = eig(K,M);
eigval = eig(K,M);
eigV = eigV(:,1:no_modes);

M_mod = eigV'*M*eigV;

K_mod = eigV'*K*eigV;

D_mod = eigV'*D*eigV;

T = [eigV(:,1:no_modes) , zeros(size(eigV(:,1:no_modes)));zeros(size(eigV(:,1:no_modes)))...
    eigV(:,1:no_modes)];

A = [zeros(size(M_mod)) eye(size(M_mod));-K_mod -D_mod];
phi_true = expm(A*dt);


%%% INPUT MATRIX FOR WAVE DISTURBANCES %%%
B_waves = [zeros(size(M_mod,1),size(tau_waves,1));eigV'*b1];
gamma_waves = A\(phi_true - eye(size(phi_true)))*B_waves;
%gamma_waves(:,end-1) = 0; % set the force contr for top node to zero, as it is accounted for in tau(:,i)


%%% INPUT MATRIX FOR 3P DISTURBANCE %%%
B_3p = [zeros(size(M_mod,1),size(tau_3P,1));eigV'*b2];
gamma_3p = A\(phi_true - eye(size(phi_true)))*B_3p;


%%% INPUT MATRIX FOR WIND THRUST %%%
B = [zeros(size(M_mod,1),no_input);eigV'*b];
gamma_known = (A)\(phi_true - eye(size(phi_true)))*B;


%%%% INPUT MATRIX FOR INDUCED BENDING MOMENT %%%%%
b3 = zeros(size(M,1),1);
b3(end) = 1;
B_mom = [zeros(size(M_mod,1),1);eigV'*b3];
gamma_mom = A\(phi_true - eye(size(phi_true)))*B_mom;


%OUTPUT FOR TRUE MODEL USED IN ESTIMATR NO.1
J=C*(eigV*eigV')*b;

J = [J;zeros(n_meas-n_meas2,size(J,2))];

G = [-C*eigV*K_mod -C*eigV*D_mod];
G = [G;[c2*eigV, zeros(n_meas-n_meas2,no_modes)]];

%OUTPUT FOR TRUE MODEL USED IN ESTIMATOR NO.2
c3 = zeros(1,size(C,2)); % added displacement measurement for bias estimator
c3(2*31+1) = 1;
C2 = [C;c3];
Gta = [-C2*eigV*K_mod -C2*eigV*D_mod]; % acc output
Gtd = [C2*eigV,zeros(size(C2*eigV))]; % displacement output


%% CALCULATING NOISECOVARIANCE MATRIX Q
q1  = [zeros(size(M_mod,1),size(tau_waves,1));eigVest'*b1];
q2 = A_est1\(phiest1 - eye(size(phiest1)))*q1;

q3 = [zeros(size(M_mod,1),size(tau_3P,1));eigVest'*b2];
q4 = A_est1\(phiest1 - eye(size(phiest1)))*q3;

Q1 = 10^5*[q2,q3]*[q2,q3]';


%% ESTIMATOR MODEL NO.2
A_est = [A_est1,[zeros(size(M_mod,1),no_modes);-1*eye(size(M_mod,1),no_modes)];[zeros(no_modes,size(A,2)),-0.1*eye(size(M_mod,1),no_modes)]];

phiest2 = expm(A_est*dt);

% process disturbance matrix for estimator model no.2
Eb = eye(size(phiest2));
Eb(1:2*no_modes,:) = 0;

Ebd = A_est\(phiest2 -eye(size(phiest2)))*Eb;

% OUTPUT MATRICES FOR ESTIMATOR NO.2
[A_est1,eigVest,G_est2,J_est2,T_est2,eigval_est2,K_mest,D_mest] = estimator_discrp(M,K,D,C,b,no_modes,[]);
Ged = [C2*eigVest,zeros(size(C2*eigVest))];
Gea = [-C2*eigVest*K_mest -C2*eigVest*D_mest];

Hd = [Ged zeros(n_meas2+1,no_modes)];
Ha = [Gea zeros(n_meas2+1,no_modes)];

%% ESTIMATOR VARIABLES 
% ESTIMATOR NO.1
var_R = 10^-3;
R1=var_R*eye(size(n_meas));
Pkk1= 10^-10*eye(size(A));
Qukk = [gamma_waves, gamma_3p];
%Q1 = Qukk*Qukk'*10^5;
Q1 = 10^5*eye(size(A));

% ESTIMATOR NO.2
var_R = 10^-5;
R2=var_R*eye(size(n_meas));
Q2 = 10^5*eye(size(phiest2));
% Q2 = [Q1,zeros(2*no_modes,no_modes);zeros(no_modes,3*no_modes)];
% for i = 2*no_modes+1:3*no_modes
% Q2(i,i) =10^5;
% end


Pkk2 = 10^-1*eye(size(phiest2));

 Q2(1:no_modes,:) = 10^-2*Q2(1:no_modes,:);
 Q2(1:9,:) = 0;
%% INITALIZATION OF VARYING PLOTTING VARIABLES
xpred1 = zeros(size(A,1),length(t));
xhatp1 = zeros(size(A,1),1);
xest1 = zeros(size(A,1),length(t));
xpred2 = zeros(size(phiest2,1),length(t));
xest2 = zeros(size(phiest2,1),length(t));
xest2(:,1) = 1;
xt = zeros(size(A,1),length(t)); %true states 

n_meas2 = n_meas2 + 1;
yinv1 = zeros(n_meas,length(t));
yinv2 = zeros(n_meas2,length(t));
y1 = zeros(n_meas,length(t));
y2 = zeros(n_meas2,length(t));
ytrue1 = zeros(n_meas,length(t));
ytrue2 = zeros(n_meas2,length(t));
taukk = zeros(no_input,length(t));

n = 1;
y_mean1 = zeros(n_meas,1);
std1=zeros(n_meas,1);
y_mean2 = zeros(n_meas2,1);
std2=zeros(n_meas2,1);

tauw = zeros(1,length(t));

for i = 1:length(t)-1

  x = T*xt(:,i); % extract moment arm
tvel = x(end-1);
 bmoment(i) = -nac_weight*9.981*x(size(M,1)-1);
    
 tauw(i) = 0.5*rho_a*A_blades*C_T*(uw(i) - tvel)*abs(uw(i) - tvel);

 tau = [tauw(i);tau_waves(end-1,i)];
tau = tauw(i);

  xt(:,i+1) = phi_true*xt(:,i) + gamma_known*tau+ gamma_waves*(tau_waves(:,i)+tau_curr(:,i)) + gamma_3p*tau_3P(:,i) + 0*gamma_mom*bmoment(i);
  
   [std1,y_mean1] = stddev_ymeas2(y1,n,dt,n_meas); 
   [std2,y_mean2] = stddev_ymeas(y2,n,dt,n_meas2); 
n_int1 = diag(std1)*0.05;
 n_int2 = diag(std2)*0.05;
 
    y1(:,i) = G*xt(:,i) + J*tau +  n_int1*vnoise(:,i);
    y2(:,i) = Gtd*xt(:,i) + n_int2*vnoise2(:,i) ;

   n = n+1; 
   
%%% UPDATE EQUATIONS FOR ESTIMATOR NO.1
Rk = G_est*Pkk1*G_est' + R1;
Mk =inv(J_est'*inv(Rk)*J_est)*J_est'*inv(Rk);
taukk(:,i) = Mk*(y1(:,i) - G_est*xhatp1);
Ppkk = inv((J_est'/(Rk)*J_est));
Lk = Pkk1*G_est'/(Rk);
yinv1 = (J*tau+n_int1*vnoise(:,i)-J_est*taukk(:,i)+(G*xt(:,i)-G_est*xhatp1));
xpred1(:,i) = xhatp1 + Lk*yinv1;
Pkk1 = Pkk1 - Lk*(Rk - J_est*Ppkk*J_est')*Lk';
Pxp = -Lk*J_est*Ppkk;
xest1(:,i) = phiest1*xpred1(:,i)+ gamma_est*(taukk(:,i));
xhatp1 = xest1(:,i);
Pkk1 = [phiest1 gamma_est]*[Pkk1 Pxp;Pxp' Ppkk]*[phiest1';gamma_est'] + Ebd1*Q1*Ebd1';
%%%%

 %%% UPDATE EQUATIONS FOR ESTIMATOR NO.2
xpred2(:,i+1) = phiest2*xest2(:,i);
S = Hd*Pkk2*Hd' + R2;
L = Pkk2*Hd'/(S);  
Pkk2 = (eye(size(phiest2,1)) - L*Hd)*Pkk2*(eye(size(phiest2,1)) - L*Hd)' + L*R2*L';
yinv2(:,i) = y2(:,i) - Hd*xpred2(:,i+1);
xest2(:,i+1) = xpred2(:,i+1) + L*yinv2(:,i);
Pkk2 = phiest2*Pkk2*phiest2' + Ebd*Q2*Ebd';




end


xtny = zeros(size(M,1)*2,length(t));
xest1ny = zeros(size(M,1)*2,length(t));
xest2ny = zeros(size(M,1)*2,length(t));

for i = 1:length(t)
    

xtny(:,i) = T*xt(:,i);
xest1ny(:,i) = T_est1*xest1(:,i);
xest2ny(:,i) = T_est2*xest2(1:2*no_modes,i);

end

xtny = xtny';
xest1ny = xest1ny';
xest2ny = xest2ny';



 pplot = 50/dt:5:length(t)-150/dt;
 
 
n58 = 2*57+1;
n58vel = size(M,1) + 2*57 +1;
n58rot = 2*58;
n58rotvel = size(M,1) + 2*58;
n26 = 2*25+1;
n26vel = size(M,1) + 2*25 +1;
n26rot = 2*26;
n26rotvel = size(M,1) + 2*26;
ttop = size(M,1)-1;
ttopvel = size(xtny,2) -1;




%% PLOTTING SECTION
restrue = struct();
restrue.tr_d = xtny(:,1:2:size(M,1));
restrue.r_d = xtny(:,2:2:size(M,1));
restrue.tr.v = xtny(:,size(M,1)+1:2:2*size(M,1));
restrue.r.v= xtny(:,size(M,1)+2:2*size(M,1));


resest1 = struct();
resest1.tr_d = xest1ny(:,1:2:size(M,1));
resest1.r_d = xest1ny(:,2:2:size(M,1));
resest1.tr.v = xest1ny(:,size(M,1)+1:2:2*size(M,1));
resest1.r.v = xest1ny(:,size(M,1)+2:2:2*size(M,1));

resest2 = struct();
resest2.tr_d = xest2ny(:,1:2:size(M,1));
resest2.r_d = xest2ny(:,2:2:size(M,1));
resest2.tr.v = xest2ny(:,size(M,1)+1:2:2*size(M,1));
resest2.r.v = xest2ny(:,size(M,1)+2:2:2*size(M,1));




%  figure(99)
%  subplot(211)
% plot(t(pplot),restrue.tr_d(pplot,58),'b')
% axis tight
% hold on
% plot(t(pplot),resest1.tr_d(pplot,58),'r');
% plot(t(pplot),resest2.tr_d(pplot,58),'k','Linewidth',1.05);
% axis tight
% title('Transversal displacement, node 58','Fontsize',14);
% legend('True','Inp-state-est','Bias-est');
% ylabel('Displacement [m]','Fontsize',14);
% subplot(212)
% axis tight
% plot(t(pplot),restrue.tr.v(pplot,58),'b');
% hold on
% plot(t(pplot),resest1.tr.v(pplot,58),'r');
% plot(t(pplot),resest2.tr.v(pplot,58),'k','Linewidth',1.05);
% axis tight
%  xlabel('Time [s]','Fontsize',14);
%  ylabel('Velocity [m/s]','Fontsize',14);
%  str=('Transversal velocity, node 58');
%  title(str,'Fontsize',14);
 
 
 figure(100)
 subplot(211)
plot(t(pplot),xtny(pplot,n26),'b')
axis tight
hold on
plot(t(pplot),xest1ny(pplot,n26),'r');
plot(t(pplot),xest2ny(pplot,n26),'k','Linewidth',1.05);
axis tight
title('Transversal displacement, node 58','Fontsize',14);
legend('True','Inp-state-est','Bias-est');
ylabel('Displacement [m]','Fontsize',14);
subplot(212)
axis tight
plot(t(pplot),xtny(pplot,n26),'b');
hold on
plot(t(pplot),xest1ny(pplot,n26),'r');
plot(t(pplot),xest2ny(pplot,n26),'k','Linewidth',1.05);
axis tight
 xlabel('Time [s]','Fontsize',14);
 ylabel('Velocity [m/s]','Fontsize',14);
 str=('Transversal velocity, node 58');
 title(str,'Fontsize',14);
 
 figure(2)
 plot(t(pplot),10^-3*taukk(1,pplot),'r');
 hold on
 plot(t(pplot),10^-3*tauw(pplot),'b');
 axis([50 150 -1.3*10^-3*max(tauw(:)) 1.3*10^-3*max(tauw(:))]);
 title('Estimated and true Wind thrust force','Fontsize',14);
 xlabel('Time [s]','Fontsize',14);
 ylabel('Force [kN]','Fontsize',14);
 legend('estimate ','true');

% figure(3)
% subplot(211)
% plot(t(pplot),10^-3*tauw(pplot),'b');
% hold on
% plot(t(pplot),10^-3*taukk(1,pplot),'r');
% title('True and estimated thrust force top node','Fontsize',14);
% ylabel('Force [kN]','Fontsize',14);
% legend('true','est/corr');
% axis tight
% subplot(212)
% plot(t(pplot),10^-4*taukk(2,pplot),'r');
% title('Corrective Force node 41','Fontsize',14);
% xlabel('Time [s]','Fontsize',14);
% ylabel('Force [kN]','Fontsize',14);
% axis tight

% figure(4)
% plot(t(pplot),10^-3*tauw(pplot),'b');
% hold on
% plot(t(pplot),10^-3*taukk(1,pplot),'r');
% title('True and estimated thrust force top node','Fontsize',14);
% ylabel('Force [kN]','Fontsize',14);
% legend('true','est/corr');
% axis tight

%%

pos = lel_mono1/2; % 1 meter above mudlevel 




el_no = floor(pos/lel_mono1)+N_mono3 + N_mono2;

  
node_val1 = [zeros(length(t),1), restrue.tr_d(:,el_no),restrue.r_d(:,el_no),...
    zeros(length(t),1),restrue.tr_d(:,el_no+1),restrue.r_d(:,el_no+1)];


node_valEst1 = [zeros(length(t),1), resest1.tr_d(:,el_no),resest1.r_d(:,el_no),...
    zeros(length(t),1),resest1.tr_d(:,el_no+1),resest1.r_d(:,el_no+1)];

node_valEst2 = [zeros(length(t),1), resest2.tr_d(:,el_no),resest2.r_d(:,el_no),...
    zeros(length(t),1),resest2.tr_d(:,el_no+1),resest2.r_d(:,el_no+1)];

%plot(t,resest2.r_d(:,el_no),'r',t,restrue.r_d(:,el_no),'b',t,resest1.r_d(:,el_no+1),'--k');

Vtrue_N10t = zeros(length(t),1);
Mtrue_10Nt = zeros(length(t),1);
Vest_N10t1 = zeros(length(t),1);
Mest_10Nt1 = zeros(length(t),1);
Vest_N10t2 = zeros(length(t),1);
Mest_10Nt2 = zeros(length(t),1);


B_N = zeros(length(t),4);
xi_N = zeros(length(t),1);


for i = 1:length(t)

Vtrue_N10t(i) = linpol_trresponse(pos,lel_mono1,node_val1(i,:));
[Mtrue_10Nt(i),B_N(i,:),xi_N(i)] = bending_moment(node_val1(i,:),pos,lel_mono1,w_radius,I,el_no);

Vest_N10t1(i) = linpol_trresponse(pos,lel_mono1,node_valEst1(i,:));
[Mest_10Nt1(i),B_N(i,:),xi_N(i)] = bending_moment(node_valEst1(i,:),pos,lel_mono1,w_radius,I,el_no);

Vest_N10t2(i) = linpol_trresponse(pos,lel_mono1,node_valEst2(i,:));
[Mest_10Nt2(i),B_N(i,:),xi_N(i)] = bending_moment(node_valEst2(i,:),pos,lel_mono1,w_radius,I,el_no);



end





figure(46)
subplot(311)
plot(t(pplot),xtny(pplot,n26),'b');
axis tight
hold on
plot(t(pplot),xest1ny(pplot,n26),'-r');
plot(t(pplot),xest2ny(pplot,n26),'-k','Linewidth',1.05);
axis tight
legend('True','Inp-state-est','Bias-est');
ylabel('Displacement [m]','Fontsize',14);
%str = sprintf('Tower top displacement from combined loading, Umean = %d [m] tint = %.2f [s]',U_10min,t_int);
str=('Transversal displacement, node 26');
title(str,'Fontsize',14);
subplot(312)
plot(t(pplot),xtny(pplot,n26vel),'b');
axis tight
hold on
plot(t(pplot),xest1ny(pplot,n26vel),'-r');
plot(t(pplot),xest2ny(pplot,n26vel),'-k','Linewidth',1.05);
axis tight
%plot(t(pplot),xpredny(pplot,node26vel),'--k');

ylabel('Velocity [m/s]','Fontsize',14);
%str = sprintf('Tower top displacement from combined loading, Umean = %d [m] tint = %.2f [s]',U_10min,t_int);
str=('Transversal velocity, node 26');
title(str,'Fontsize',14);

mw = 4*10^3/(I(1)*10^6);

subplot(313)
plot(t(pplot),Mtrue_10Nt(pplot)*10^-3,'b');
axis tight
hold on
plot(t(pplot),Mest_10Nt1(pplot)*10^-3,'-r');
plot(t(pplot),Mest_10Nt2(pplot)*10^-3,'-k','Linewidth',1.05);
axis tight

title('Mudline bending moment','Fontsize',14);
xlabel('Time [s]','Fontsize',14);
ylabel('kNm','Fontsize',14);

% 
%  figure(3)
%  subplot(311)
%  plot(t,tau_waves(end-2,:)*10^-3,'r');
% title('Wave loading sea surface Hm0 = 1 [m] Tp = 5.00 [s]','Fontsize',14);
% ylabel('Force [kN]','Fontsize',14);
% subplot(312)
% plot(t,-xtny(:,size(M,1)-1),'b')
% ylabel('Displacement [m]','Fontsize',14);
% title('Towertop displacement','Fontsize',14);
% subplot(313)
% plot(t,Mtrue_10Nt(:)*10^-3,'k');
% xlabel('Time [sec]','Fontsize',14);
% ylabel('kNm','Fontsize',14);
% title('Mudline bending moment','Fontsize',14);
% 
%  figure(5)
%  pimp = 50/dt:1:120/dt;
%  my = size(M,1)+2*40+1;
% plot(t(pimp),xtny(pimp,my));
% xlabel('Time [sec]','Fontsize',14);
% ylabel('Velocity [m/s]','Fontsize',14);
% axis([pimp(1)*dt pimp(end)*dt -1.3*max(xtny(pimp,my)) 1.3*max(xtny(pimp,my))]);
% title('Horizontal velocity at sea surface node Hm0=5 Tp=11 Umean=11 [m/s]','Fontsize',14);
% 




